<?php
namespace PushAlert\PushNotifications\Observer;

use Magento\Framework\Event\ObserverInterface;

/**
 * Handles Abandoned Cart and Stock/Inventory Notifications
 *
 * @author Abhinav
 */
class AbandonedCartObserver implements ObserverInterface
{
    private $customerSession;
    private $urlInterface;
    private $orderDetails;
    private $_curl;
    private $cookieManager;
    private $cookieMetadataFactory;
    private $helper;

    public function __construct(
        \PushAlert\PushNotifications\Helper\PushAlertHelper $helper,
        \Magento\Checkout\Model\Cart $cart,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Sales\Model\Order $orderDetails,
        \Magento\Framework\HTTP\Client\Curl $curl,    
        \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory $cookieMetadataFactory,
        \Magento\Framework\Stdlib\CookieManagerInterface $cookieManager
    ) {
        $this->helper = $helper;
        $this->cart = $cart;
        $this->customerSession = $customerSession;
        $this->urlInterface = $urlInterface;
        $this->orderDetails = $orderDetails;
        $this->_curl=$curl;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->cookieManager = $cookieManager;
    }

    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $isEnabled = $this->helper->isEnabled();
        $abandonedCartEnabled = $this->helper->getConfig("corepushalert/settings/_pushalert_abandoned_cart");

        $eventName = $observer->getEvent()->getName();

        if ($isEnabled && $abandonedCartEnabled) {
            if ($eventName == "checkout_cart_save_after") {
                $this->addedCart();
            } elseif ($eventName == "checkout_onepage_controller_success_action") {
                $this->checkOrderCompleted($observer);
            }
        }
        
        if ($isEnabled) {
            if ($eventName == "checkout_cart_product_add_after" || $eventName == "checkout_onepage_controller_success_action") {
                $this->paBrowseAbandonmentCompleted();
            }
        }
    }
    
    public function paBrowseAbandonmentCompleted(){
        $cookieMetadata = $this->cookieMetadataFactory->createPublicCookieMetadata()
                                    ->setDurationOneYear()
                                    ->setPath("/");
            
        $cookie_id = $this->helper->getConfig('corepushalert/settings/_pushalert_cookie_id');
        $this->cookieManager->setPublicCookie('pa_oc_' . $cookie_id . 'completed', '1', $cookieMetadata);
    }

    private function checkOrderCompleted(\Magento\Framework\Event\Observer $observer)
    {
        try {
            
            $check_pushalert_mage = filter_input(INPUT_COOKIE, 'pushalert_mage');
            $check_pushalert_mage = isset($check_pushalert_mage) ? $check_pushalert_mage : false;
            $orderIds = $observer->getEvent()->getOrderIds();
            $order = $this->orderDetails->load($orderIds[0]);
            if ($check_pushalert_mage) {
                $order_id = $order->getRealOrderId();
                $this->pushalertTrackOrder($order_id, $order->getGrandTotal(), $check_pushalert_mage);
            }
        } catch (\Exception $e) {
            return false;
        }
        $this->paClearAbandonedCart();
    }

    private function pushalertTrackOrder($order_id, $order_total, $check_pushalert_mage)
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/order";

//POST variables
            $post_vars = [
                "order_id" => $order_id,
                "order_total" => $order_total,
                "source" => $check_pushalert_mage
            ];

            if (!$this->helper->backgroundPost($curlUrl . "?" . http_build_query($post_vars))) {
                /*$headers = [];
                $headers[] = "Authorization: api_key=" . $apiKey;

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $curlUrl);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                $result = curl_exec($ch);*/
                $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
                $this->_curl->post($curlUrl, $post_vars);
                $response = json_decode($this->_curl->getBody(), true);
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function addedCart()
    {
        try {
            if (!$this->helper->isEnabled()) {
                return;
            }
            
            $total_items = 0;
            $cart = $this->cart->getQuote();
            $total_items = $cart->getItemsCount();
            if ($total_items > 0) {
                $this->paInitAbandonedCart($total_items);
            } else {
                $this->paClearAbandonedCart();
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function paInitAbandonedCart($total_items)
    {
        try {
            $url = $this->urlInterface->getUrl('checkout');
            $cookie_id = $this->helper->getConfig('corepushalert/settings/_pushalert_cookie_id');
            $pushalert_subs_id = filter_input(INPUT_COOKIE, 'pushalert_' . $cookie_id . 'subs_id');
            $user_info = [];
            $user_info['cart_url'] = $url;
            $user_info['checkout_url'] = $user_info['cart_url'];
            $user_info['total_items'] = $total_items;
            $curr_user_id = $this->helper->getCustomerID();

            if ($curr_user_id != null || $curr_user_id != 0) {
                $user_info['first_name'] = $this->helper->getCustomerFirstName();
                $this->pushalertAddAbandonedCart($curr_user_id, $user_info);
            } else {
                $this->pushalertAddAbandonedCart($pushalert_subs_id, $user_info);
            }
        } catch (\Exception $e) {
            return false;
        }
    }

    private function pushalertAddAbandonedCart($subscriber_id, $user_info = [])
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }
            //file_put_contents("new-file.txt", "Inside Abandoned Cart Final");

            $curlUrl = "https://api.pushalert.co/rest/v1/abandonedCart";

            //POST variables
            $post_vars = [
                "subscriber" => $subscriber_id,
                "extra_info" => json_encode($user_info)
            ];
            if (!$this->helper->backgroundPost($curlUrl . "?" . http_build_query($post_vars))) {
                /*$headers = [];
                $headers[] = ["Authorization"=>"api_key=".$apiKey];

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $curlUrl);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                $result = curl_exec($ch);
                $result = json_decode($result, true);*/
                $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
                $this->_curl->post($curlUrl, $post_vars);
                $response = json_decode($this->_curl->getBody(), true);
            }
        } catch (\Exception $e) {
            return false;
            //$this->_logger->addInfo("Add to cart: " . $e->getMessage);
        }
    }

    private function paClearAbandonedCart()
    {
        try {
            if (!$this->helper->paSubscriptionCheck()) {
                return;
            }

            $curr_user_id = $this->helper->getCustomerID();
            $cookie_id = $this->helper->getConfig('corepushalert/settings/_pushalert_cookie_id');
            $pushalert_subs_id = filter_input(INPUT_COOKIE, 'pushalert_' . $cookie_id . 'subs_id');

            if ($curr_user_id != 0) {
                $this->pushalertRemoveAbandonedCart($curr_user_id);
            }

            $this->pushalertRemoveAbandonedCart($pushalert_subs_id);
        } catch (\Exception $e) {
            return false; //$this->_logger->addInfo("Clear Abandoned Cart: " . $e->getMessage);
        }
    }

    private function pushalertRemoveAbandonedCart($subscriber_id)
    {
        try {
            $apiKey = $this->helper->getRestAPIKey();
            if (!$apiKey) {
                return false;
            }

            $curlUrl = "https://api.pushalert.co/rest/v1/abandonedCart/delete";

            $post_vars = [
                "subscriber" => $subscriber_id
            ];

            if (!$this->helper->backgroundPost($curlUrl . "?" . http_build_query($post_vars))) {
                /*$headers = [];
                $headers[] = "Authorization: api_key=" . $apiKey;

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $curlUrl);
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post_vars));
                curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                $result = curl_exec($ch);
                $result = json_decode($result, true);*/
                $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                $this->_curl->addHeader("Authorization", "api_key=".$apiKey);
                $this->_curl->post($curlUrl, $post_vars);
                $response = json_decode($this->_curl->getBody(), true);
            }
        } catch (\Exception $e) {
            return false;
            //$this->_logger->addInfo("Remove Abandoned Cart: " . $e->getMessage);
        }
    }
}
